@@ -28,7 +28,6 @@
 #include "lchown.h"
 #include "group-member.h"
 #include "quote.h"
-#include "savedir.h"
 #include "xstrtol.h"
 #include "chown-core.h"
 
@@ -129,24 +128,28 @@ Usage: %s [OPTION]... GROUP FILE...\n\
   or:  %s [OPTION]... --reference=RFILE FILE...\n\
 "),
 	      program_name, program_name);
-      printf (_("\
+      fputs (_("\
 Change the group membership of each FILE to GROUP.\n\
 \n\
   -c, --changes          like verbose but report only when a change is made\n\
       --dereference      affect the referent of each symbolic link, rather\n\
                          than the symbolic link itself\n\
+"), stdout);
+      fputs (_("\
   -h, --no-dereference   affect symbolic links instead of any referenced file\n\
                          (available only on systems that can change the\n\
                          ownership of a symlink)\n\
+"), stdout);
+      fputs (_("\
   -f, --silent, --quiet  suppress most error messages\n\
       --reference=RFILE  use RFILE's group rather than the specified\n\
                          GROUP value\n\
   -R, --recursive        operate on files and directories recursively\n\
   -v, --verbose          output a diagnostic for every file processed\n\
-      --help             display this help and exit\n\
-      --version          output version information and exit\n\
-"));
-      puts (_("\nReport bugs to <bug-fileutils@gnu.org>."));
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -212,7 +215,8 @@ main (int argc, char **argv)
     {
       struct stat ref_stats;
       if (stat (reference_file, &ref_stats))
-	error (1, errno, _("getting attributes of %s"), quote (reference_file));
+	error (1, errno, _("failed to get attributes of %s"),
+	       quote (reference_file));
 
       chopt.group_name = gid_to_name (ref_stats.st_gid);
       gid = ref_stats.st_gid;
