@@ -1,5 +1,5 @@
 /* mv -- move or rename files
-   Copyright (C) 86, 89, 90, 91, 1995-2001 Free Software Foundation, Inc.
+   Copyright (C) 86, 89, 90, 91, 1995-2002 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,9 +28,11 @@
 #include <assert.h>
 
 #include "system.h"
+#include "argmatch.h"
 #include "backupfile.h"
 #include "copy.h"
 #include "cp-hash.h"
+#include "dirname.h"
 #include "error.h"
 #include "path-concat.h"
 #include "quote.h"
@@ -39,7 +41,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "mv"
 
-#define AUTHORS "Mike Parker, David MacKenzie, and Jim Meyering"
+#define AUTHORS N_ ("Mike Parker, David MacKenzie, and Jim Meyering")
 
 /* Initial number of entries in each hash table entry's table of inodes.  */
 #define INITIAL_HASH_MODULE 100
@@ -52,14 +54,12 @@
 enum
 {
   TARGET_DIRECTORY_OPTION = CHAR_MAX + 1,
-  STRIP_TRAILING_SLASHES_OPTION
+  STRIP_TRAILING_SLASHES_OPTION,
+  REPLY_OPTION
 };
 
-int euidaccess ();
-int full_write ();
 int isdir ();
 int lstat ();
-int yesno ();
 
 /* The name this program was run with. */
 char *program_name;
@@ -67,11 +67,24 @@ char *program_name;
 /* Remove any trailing slashes from each SOURCE argument.  */
 static int remove_trailing_slashes;
 
+/* Valid arguments to the `--reply' option. */
+static char const* const reply_args[] =
+{
+  "yes", "no", "query", 0
+};
+
+/* The values that correspond to the above strings. */
+static int const reply_vals[] =
+{
+  I_ALWAYS_YES, I_ALWAYS_NO, I_ASK_USER
+};
+
 static struct option const long_options[] =
 {
   {"backup", optional_argument, NULL, 'b'},
   {"force", no_argument, NULL, 'f'},
   {"interactive", no_argument, NULL, 'i'},
+  {"reply", required_argument, NULL, REPLY_OPTION},
   {"strip-trailing-slashes", no_argument, NULL, STRIP_TRAILING_SLASHES_OPTION},
   {"suffix", required_argument, NULL, 'S'},
   {"target-directory", required_argument, NULL, TARGET_DIRECTORY_OPTION},
@@ -107,14 +120,14 @@ cp_option_init (struct cp_options *x)
   x->dereference = DEREF_NEVER;
   x->unlink_dest_before_opening = 0;
   x->unlink_dest_after_failed_open = 0;
-  x->failed_unlink_is_fatal = 1;
   x->hard_link = 0;
-  x->interactive = 0;
+  x->interactive = I_UNSPECIFIED;
   x->move_mode = 1;
   x->myeuid = geteuid ();
   x->one_file_system = 0;
-  x->preserve_owner_and_group = 1;
-  x->preserve_chmod_bits = 1;
+  x->preserve_ownership = 1;
+  x->preserve_links = 1;
+  x->preserve_mode = 1;
   x->preserve_timestamps = 1;
   x->require_preserve = 0;  /* FIXME: maybe make this an option */
   x->recursive = 1;
@@ -122,6 +135,7 @@ cp_option_init (struct cp_options *x)
   x->symbolic_link = 0;
   x->set_mode = 0;
   x->mode = 0;
+  x->stdin_tty = isatty (STDIN_FILENO);
 
   /* Find out the current file creation mask, to knock the right bits
      when using chmod.  The creation mask is set to be liberal, so
@@ -132,6 +146,8 @@ cp_option_init (struct cp_options *x)
   x->update = 0;
   x->verbose = 0;
   x->xstat = lstat;
+  x->dest_info = NULL;
+  x->src_info = NULL;
 }
 
 /* If PATH is an existing directory, return nonzero, else 0.  */
@@ -144,18 +160,6 @@ is_real_dir (const char *path)
   return lstat (path, &stats) == 0 && S_ISDIR (stats.st_mode);
 }
 
-static int
-strip_trailing_slashes_2 (char *path)
-{
-  char *end_p = path + strlen (path) - 1;
-  char *slash = end_p;
-
-  while (slash > path && *slash == '/')
-    *slash-- = '\0';
-
-  return slash < end_p;
-}
-
 /* Move SOURCE onto DEST.  Handles cross-filesystem moves.
    If SOURCE is a directory, DEST must not exist.
    Return 0 if successful, non-zero if an error occurred.  */
@@ -173,14 +177,14 @@ do_move (const char *source, const char *dest, const struct cp_options *x)
       first = 0;
 
       /* Allocate space for remembering copied and created files.  */
-      hash_init (INITIAL_HASH_MODULE, INITIAL_ENTRY_TAB_SIZE);
+      hash_init ();
     }
 
   fail = copy (source, dest, 0, x, &copy_into_self, &rename_succeeded);
 
   if (!fail)
     {
-      const char *dir_to_remove;
+      char const *dir_to_remove;
       if (copy_into_self)
 	{
 	  /* In general, when copy returns with copy_into_self set, SOURCE is
@@ -200,7 +204,7 @@ do_move (const char *source, const char *dest, const struct cp_options *x)
       else if (rename_succeeded)
 	{
 	  /* No need to remove anything.  SOURCE was successfully
-	     renamed to DEST.  */
+	     renamed to DEST.  Or the user declined to rename a file.  */
 	  dir_to_remove = NULL;
 	}
       else
@@ -233,29 +237,15 @@ do_move (const char *source, const char *dest, const struct cp_options *x)
       if (dir_to_remove != NULL)
 	{
 	  struct rm_options rm_options;
-	  struct File_spec fs;
 	  enum RM_status status;
 
 	  rm_option_init (&rm_options);
 	  rm_options.verbose = x->verbose;
 
-	  remove_init ();
-
-	  fspec_init_file (&fs, dir_to_remove);
-
-	  /* Remove any trailing slashes.  This is necessary if we
-	     took the else branch of movefile.  */
-	  strip_trailing_slashes_2 (fs.filename);
-
-	  status = rm (&fs, 1, &rm_options);
+	  status = rm (1, &dir_to_remove, &rm_options);
 	  assert (VALID_STATUS (status));
 	  if (status == RM_ERROR)
 	    fail = 1;
-
-	  remove_fini ();
-
-	  if (fail)
-	    error (0, errno, _("cannot remove %s"), quote (dir_to_remove));
 	}
     }
 
@@ -271,7 +261,7 @@ static int
 movefile (char *source, char *dest, int dest_is_dir,
 	  const struct cp_options *x)
 {
-  int dest_had_trailing_slash = strip_trailing_slashes_2 (dest);
+  int dest_had_trailing_slash = strip_trailing_slashes (dest);
   int fail;
 
   /* This code was introduced to handle the ambiguity in the semantics
@@ -282,7 +272,7 @@ movefile (char *source, char *dest, int dest_is_dir,
      rename semantics are POSIX and susv2 compliant.  */
 
   if (remove_trailing_slashes)
-    strip_trailing_slashes_2 (source);
+    strip_trailing_slashes (source);
 
   /* In addition to when DEST is a directory, if DEST has a trailing
      slash and neither SOURCE nor DEST is a directory, presume the target
@@ -297,18 +287,13 @@ movefile (char *source, char *dest, int dest_is_dir,
       char *src_basename;
       char *new_dest;
 
-      /* Remove trailing slashes before taking base_name.
-	 Otherwise, base_name ("a/") returns "".  */
-      strip_trailing_slashes_2 (source);
-
+      strip_trailing_slashes (source);
       src_basename = base_name (source);
       new_dest = path_concat (dest, src_basename, NULL);
       if (new_dest == NULL)
 	xalloc_die ();
       fail = do_move (source, new_dest, x);
-
-      /* Do not free new_dest.  It may have been squirreled away by
-	 the remember_copied function.  */
+      free (new_dest);
     }
   else
     {
@@ -332,34 +317,51 @@ Usage: %s [OPTION]... SOURCE DEST\n\
   or:  %s [OPTION]... --target-directory=DIRECTORY SOURCE...\n\
 "),
 	      program_name, program_name, program_name);
-      printf (_("\
+      fputs (_("\
 Rename SOURCE to DEST, or move SOURCE(s) to DIRECTORY.\n\
 \n\
+"), stdout);
+      fputs (_("\
+Mandatory arguments to long options are mandatory for short options too.\n\
+"), stdout);
+      fputs (_("\
       --backup[=CONTROL]       make a backup of each existing destination file\n\
   -b                           like --backup but does not accept an argument\n\
-  -f, --force                  never prompt before overwriting\n\
+  -f, --force                  do not prompt before overwriting\n\
+                                 equivalent to --reply=yes\n\
   -i, --interactive            prompt before overwrite\n\
-      --strip-trailing-slashes  remove any trailing slashes from each SOURCE\n\
+                                 equivalent to --reply=query\n\
+"), stdout);
+      fputs (_("\
+      --reply={yes,no,query}   specify how to handle the prompt about an\n\
+                                 existing destination file\n\
+      --strip-trailing-slashes remove any trailing slashes from each SOURCE\n\
                                  argument\n\
   -S, --suffix=SUFFIX          override the usual backup suffix\n\
+"), stdout);
+      fputs (_("\
       --target-directory=DIRECTORY  move all SOURCE arguments into DIRECTORY\n\
-  -u, --update                 move only older or brand new non-directories\n\
+  -u, --update                 move only when the SOURCE file is newer\n\
+                                 than the destination file or when the\n\
+                                 destination file is missing\n\
   -v, --verbose                explain what is being done\n\
-      --help                   display this help and exit\n\
-      --version                output version information and exit\n\
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      fputs (_("\
 \n\
-"));
-      printf (_("\
 The backup suffix is `~', unless set with --suffix or SIMPLE_BACKUP_SUFFIX.\n\
 The version control method may be selected via the --backup option or through\n\
 the VERSION_CONTROL environment variable.  Here are the values:\n\
 \n\
+"), stdout);
+      fputs (_("\
   none, off       never make backups (even if --backup is given)\n\
   numbered, t     make numbered backups\n\
   existing, nil   numbered if numbered backups exist, simple otherwise\n\
   simple, never   always make simple backups\n\
-"));
-      puts (_("\nReport bugs to <bug-fileutils@gnu.org>."));
+"), stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -414,10 +416,14 @@ main (int argc, char **argv)
 	    version_control_string = optarg;
 	  break;
 	case 'f':
-	  x.interactive = 0;
+	  x.interactive = I_ALWAYS_YES;
 	  break;
 	case 'i':
-	  x.interactive = 1;
+	  x.interactive = I_ASK_USER;
+	  break;
+	case REPLY_OPTION:
+	  x.interactive = XARGMATCH ("--reply", optarg,
+				     reply_args, reply_vals);
 	  break;
 	case STRIP_TRAILING_SLASHES_OPTION:
 	  remove_trailing_slashes = 1;
@@ -487,6 +493,13 @@ main (int argc, char **argv)
 				  ? n_files - 1
 				  : n_files - 2);
     unsigned int i;
+
+    /* Initialize the hash table only if we'll need it.
+       The problem it is used to detect can arise only if there are
+       two or more files to move.  */
+    if (last_file_idx)
+      dest_info_init (&x);
+
     for (i = 0; i <= last_file_idx; ++i)
       errors |= movefile (file[i], target_directory, dest_is_dir, &x);
   }
