@@ -1,5 +1,5 @@
 /* rmdir -- remove directories
-   Copyright (C) 90, 91, 1995-2000 Free Software Foundation, Inc.
+   Copyright (C) 90, 91, 1995-2001 Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -44,8 +44,6 @@
 # define ENOTEMPTY 0
 #endif
 
-void strip_trailing_slashes ();
-
 /* The name this program was run with. */
 char *program_name;
 
@@ -144,20 +142,22 @@ usage (int status)
   else
     {
       printf (_("Usage: %s [OPTION]... DIRECTORY...\n"), program_name);
-      printf (_("\
+      fputs (_("\
 Remove the DIRECTORY(ies), if they are empty.\n\
 \n\
       --ignore-fail-on-non-empty\n\
                   ignore each failure that is solely because a directory\n\
                   is non-empty\n\
+"), stdout);
+      fputs (_("\
   -p, --parents   remove DIRECTORY, then try to remove each directory\n\
                   component of that path name.  E.g., `rmdir -p a/b/c' is\n\
                   similar to `rmdir a/b/c a/b a'.\n\
   -v, --verbose   output a diagnostic for every directory processed\n\
-      --help      display this help and exit\n\
-      --version   output version information and exit\n\
-"));
-      puts (_("\nReport bugs to <bug-fileutils@gnu.org>."));
+"), stdout);
+      fputs (HELP_OPTION_DESCRIPTION, stdout);
+      fputs (VERSION_OPTION_DESCRIPTION, stdout);
+      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
     }
   exit (status);
 }
@@ -210,10 +210,6 @@ main (int argc, char **argv)
       int fail;
       char *dir = argv[optind];
 
-      /* Stripping slashes is harmless for rmdir;
-	 if the arg is not a directory, it will fail with ENOTDIR.  */
-      strip_trailing_slashes (dir);
-
       /* Give a diagnostic for each attempted removal if --verbose.  */
       if (verbose)
 	error (0, 0, _("removing directory, %s"), dir);
